/*
 * WS-I Sample Application 1.0
 * 
 * Copyright (c) 2005 by SAP AG. 
 * All Rights Reserved.
 * 
 */
package org.wsi.scm.test;

import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.Date;

import org.wsi.scm.core.bo.configuration.ConfigurationEndpointRole;
import org.wsi.scm.core.bo.configuration.ConfigurationEndpointType;
import org.wsi.scm.core.bo.configuration.ConfigurationType;


public abstract class AbstractTest
{
	protected AbstractTest instance;
	
	private String errorMessage = "";
	private String logName = "test";
	private Throwable cause;
	private FileOutputStream fos = null;
	protected PrintWriter log =  null;

	
	public abstract String getTestName();
	
	public abstract String getDescription();
	
	/**
	 * This method is the entry point for the test class. It should be called
	 * to start the test!
	 */
	public boolean execute() {
		long start = 0;
		long end = 0;
		try {
			start = System.currentTimeMillis();
			logName = "C:/" + getTestName() + "_" + start + ".log";
			fos = new FileOutputStream(logName);
			log = new PrintWriter(fos);
			log.println(getTestName() + " started at " + new Date(start));
			doTest();
		} catch (Exception e) {
			this.errorMessage = e.getMessage();
			cause = e.getCause();
			e.printStackTrace(log);
			end = System.currentTimeMillis();
			log.println("Stopped processing at " + new Date(end) + ", took " + (end-start) + "ms");
			log.println("Couldn't finish because of: " + e.getMessage());
			log.flush();
			log.close();
			return false;
		}
		end = System.currentTimeMillis();
		log.println("Finished at " + new Date(end) + ", took " + (end-start) + "ms");
		log.flush();
		log.close();
		return true;
	}
	
	public String getLogName() {
		return logName;
	}
	
	public String getErrorMessage() {
		return this.errorMessage;
	}
	
	public ConfigurationType getDefaultConfiguration() {

		ConfigurationType configurationHeader = new ConfigurationType();
		
		ConfigurationEndpointType retailer = new ConfigurationEndpointType();
		ConfigurationEndpointRole retailerRole = new ConfigurationEndpointRole();
		retailerRole.setRole("Retailer");
		retailer.setRole(retailerRole);
		retailer.setValue("http://localhost:53000/RetailerService/RetailerSoap?style=document");		
		
		ConfigurationEndpointType wha = new ConfigurationEndpointType();
		ConfigurationEndpointRole wharole = new ConfigurationEndpointRole();
		wharole.setRole("WarehouseA");
		wha.setRole(wharole);
		wha.setValue("http://localhost:53000/WarehouseServiceA/WarehouseSoap?style=document");

		ConfigurationEndpointType whb = new ConfigurationEndpointType();
		ConfigurationEndpointRole whbrole = new ConfigurationEndpointRole();
		whbrole.setRole("WarehouseB");
		whb.setRole(whbrole);
		whb.setValue("http://localhost:53000/WarehouseServiceB/WarehouseSoap?style=document");

		ConfigurationEndpointType whc = new ConfigurationEndpointType();
		ConfigurationEndpointRole whcrole = new ConfigurationEndpointRole();
		whcrole.setRole("WarehouseC");
		whc.setRole(whcrole);
		whc.setValue("http://localhost:53000/WarehouseServiceC/WarehouseSoap?style=document");
		
		ConfigurationEndpointType mfa = new ConfigurationEndpointType();
		ConfigurationEndpointRole mfarole = new ConfigurationEndpointRole();
		mfarole.setRole("ManufacturerA");
		mfa.setRole(mfarole);
		mfa.setValue("http://localhost:53000/ManufacturerServiceA/ManufacturerSoap?style=document");

		ConfigurationEndpointType mfb = new ConfigurationEndpointType();
		ConfigurationEndpointRole mfbrole = new ConfigurationEndpointRole();
		mfbrole.setRole("ManufacturerB");
		mfb.setRole(mfbrole);
		mfb.setValue("http://localhost:53000/ManufacturerServiceB/ManufacturerSoap?style=document");
		
		ConfigurationEndpointType mfc = new ConfigurationEndpointType();
		ConfigurationEndpointRole mfcrole = new ConfigurationEndpointRole();
		mfcrole.setRole("ManufacturerC");
		mfc.setRole(mfcrole);
		mfc.setValue("http://localhost:53000/ManufacturerServiceC/ManufacturerSoap?style=document");
		
		ConfigurationEndpointType logging = new ConfigurationEndpointType();
		ConfigurationEndpointRole loggingRole = new ConfigurationEndpointRole();
		loggingRole.setRole("LoggingFacility");
		logging.setRole(mfcrole);
		logging.setValue("http://localhost:53000/LoggingFacility/LoggingFacilitySoap?style=document");

		ConfigurationEndpointType[] endpoints = new ConfigurationEndpointType[8];
		endpoints[0] = retailer;
		endpoints[1] = logging;
		endpoints[2] = wha;	
		endpoints[3] = whb;
		endpoints[4] = whc;
		endpoints[5] = mfa;
		endpoints[6] = mfb;
		endpoints[7] = mfc;
		
		configurationHeader.setServiceUrl(endpoints);
		
		configurationHeader.setUserId("DefaultUser");
		
		return configurationHeader;
	}
	
	public Throwable getCause() {
		return cause;
	}

	public abstract void doTest() throws Exception;
}
