package org.wsi.scm.test;

import java.math.BigDecimal;
import java.math.BigInteger;

import javax.naming.Context;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.wsi.scm.proxy.demo.DemoService;
import org.wsi.scm.proxy.demo.DemoServiceViDocument;
import org.wsi.scm.proxy.demo.types.Config;
import org.wsi.scm.proxy.demo.types.ConfigData;
import org.wsi.scm.proxy.retailer.RetailerService;
import org.wsi.scm.proxy.retailer.RetailerServiceViRpc;
import org.wsi.scm.proxy.retailer.types.p1.CustomerDetails;
import org.wsi.scm.proxy.retailer.types.p1.PartsOrderItem;
import org.wsi.scm.proxy.retailer.types.p1.PartsOrder;
import org.wsi.scm.util.Constants;
import org.wsi.scm.util.ServiceLocator;

import com.sap.engine.services.webservices.jaxrpc.wsdl2java.BaseGeneratedStub;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.ProtocolList;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.features.builtin.SoapHeadersProtocol;

public class RetailerServiceTest2 extends AbstractTest
{

	public String getTestName()
	{
		return "RetailerServiceTest2";
	}

	public String getDescription()
	{
		return "Tests the custom header functions.";
	}

	public void doTest() throws Exception
	{
		Context ctx = ServiceLocator.getInstance().getContext();

		RetailerService ws =
			(RetailerService) ctx.lookup(Constants.JNDI_SERVICE_PREFIX + Constants.WSPROXY_RETAILER_SERVICE);
		RetailerServiceViRpc retailer =
			(RetailerServiceViRpc) ws.getLogicalPort("SecurePort");

		DemoService ds = (DemoService) ctx.lookup(Constants.JNDI_SERVICE_PREFIX + Constants.WSPROXY_DEMO_SERVICE);
		DemoServiceViDocument demo = (DemoServiceViDocument) ds.getLogicalPort();

		PartsOrderItem[] items = new PartsOrderItem[1];
		items[0] = new PartsOrderItem();
		items[0].setPrice(new BigDecimal(2.2));
		items[0].setProductNumber(new BigInteger("605001"));
		items[0].setQuantity(new BigInteger("1"));
		PartsOrder partsOrder = new PartsOrder();
		partsOrder.setItem(items);
		CustomerDetails customerDetails = new CustomerDetails();
		customerDetails.setCustnbr("1234");

		// workaround for re-ordering of protocols needed to enable security for SOAP headers! 
		ProtocolList protocolList = ((BaseGeneratedStub) retailer)._getGlobalProtocols();
		SoapHeadersProtocol headers =
			(SoapHeadersProtocol) ((BaseGeneratedStub) retailer)._getGlobalProtocols().getProtocol(
				"SoapHeadersProtocol");
		protocolList.remove(headers);
		protocolList.add(0, headers);

		// build config header from demo service call to getConfigFromDB
		Config[] config = demo.getConfigFromDB();

		// create header from first element in config (e.g. 'public'
		ConfigData[] configData = config[2].getConfigData();

		// root element of config header
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		Document doc = docBuilder.newDocument();
		Element parameterNode =
			doc.createElementNS(
				Constants.SOAPHEADER_CONFIG_NAMESPACE,
				Constants.SOAPHEADER_CONFIG_NAMESPACE_PREFIX + Constants.SOAPHEADER_CONFIG_TAGNAME_ROOT);

		Element valueNodeUser = doc.createElement(Constants.SOAPHEADER_CONFIG_NAMESPACE_PREFIX + "UserId");
		valueNodeUser.appendChild(doc.createTextNode("testValue"));
		parameterNode.setAttributeNS(
			"http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd",
			"wsu:Id",
			"configuration");

		for (int i = 0; i < configData.length; i++)
		{
			ConfigData endpointConfiguration = configData[i];

			Element valueNodeLoggingConfig =
				doc.createElement(
					Constants.SOAPHEADER_CONFIG_NAMESPACE_PREFIX + Constants.SOAPHEADER_CONFIG_TAGNAME_SERVICE);
			valueNodeLoggingConfig.setAttribute(
				Constants.SOAPHEADER_CONFIG_ATTRIBUTENAME_ROLE,
				endpointConfiguration.getRoleName());
			valueNodeLoggingConfig.appendChild(doc.createTextNode(endpointConfiguration.getServiceUrl()));

			parameterNode.appendChild(valueNodeUser);
			parameterNode.appendChild(valueNodeLoggingConfig);
		}

		headers.setOutputHeader(null, parameterNode);

		retailer._setProperty(
			RetailerServiceViRpc.ENDPOINT_ADDRESS_PROPERTY,
			"http://wsi.pag.directtaps.net/WsiScmSampleApplication/Retailer/Retailer.asmx");

		retailer.submitOrder(items, customerDetails);
	}
}