package org.wsi.scm.test;

import java.math.BigDecimal;
import java.math.BigInteger;

import javax.naming.Context;

import org.w3c.dom.Element;
import org.wsi.scm.core.bo.configuration.ConfigurationType;
import org.wsi.scm.proxy.retailer.RetailerService;
import org.wsi.scm.proxy.retailer.RetailerServiceViRpc;
import org.wsi.scm.proxy.retailer.types.p1.CustomerDetails;
import org.wsi.scm.proxy.retailer.types.p1.PartsOrderItem;
import org.wsi.scm.proxy.retailer.types.p1.PartsOrderResponseItem;
import org.wsi.scm.util.Constants;
import org.wsi.scm.util.DemoConfiguration;
import org.wsi.scm.util.ServiceLocator;
import org.wsi.scm.util.Util;

import com.sap.engine.services.webservices.jaxrpc.wsdl2java.BaseGeneratedStub;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.ProtocolList;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.features.builtin.SoapHeadersProtocol;

public class WSITest1 extends AbstractTest
{

	public String getTestName()
	{
		return "WSITest1";
	}

	public String getDescription()
	{
		return "Test1 - no availability for any products.";
	}

	public void doTest() throws Exception
	{
		String demoUserId = "abc123";
		
		DemoConfiguration.getInstance().storeHttpProxyHostForDemo(demoUserId, "localhost");
		DemoConfiguration.getInstance().storeHttpProxyPortForDemo(demoUserId, "50012");	

		Context ctx = ServiceLocator.getInstance().getContext();

		RetailerService ws =
			(RetailerService) ctx.lookup(Constants.JNDI_SERVICE_PREFIX + Constants.WSPROXY_RETAILER_SERVICE);
		RetailerServiceViRpc retailer = (RetailerServiceViRpc) ws.getLogicalPort("SecurePort");

		PartsOrderItem[] items = new PartsOrderItem[1];
		items[0] = new PartsOrderItem();
		items[0].setProductNumber(new BigInteger("605001"));
		items[0].setQuantity(new BigInteger("250"));
		items[0].setPrice(new BigDecimal("2.0"));

		CustomerDetails customerDetails = new CustomerDetails();
		customerDetails.setCustnbr(getTestName());

		// get configuration from demo user id
		ConfigurationType configuration = DemoConfiguration.getInstance().getConfigurationForDemo(demoUserId);

		// create header for call to retailer service
		Element configurationSoapHeader = Util.getConfigurationHeader(configuration);

		// workaround for re-ordering of protocols needed to enable security for SOAP headers! 
		ProtocolList protocolList = ((BaseGeneratedStub) retailer)._getGlobalProtocols();
		SoapHeadersProtocol retailerRequestHeaders =
			(SoapHeadersProtocol) ((BaseGeneratedStub) retailer)._getGlobalProtocols().getProtocol(
				"SoapHeadersProtocol");
		protocolList.remove(retailerRequestHeaders);
		protocolList.add(0, retailerRequestHeaders);

		// add config header
		retailerRequestHeaders.setOutputHeader(null, Util.getConfigurationHeader(configuration));
 
		retailer._setProperty("javax.xml.rpc.http.proxyhost", "localhost");
		retailer._setProperty("javax.xml.rpc.http.proxyport", "50012");

		retailer._setProperty(
			RetailerServiceViRpc.ENDPOINT_ADDRESS_PROPERTY,
			Util.getEndpointForRole(configuration, Constants.ROLE_RETAILER));

		PartsOrderResponseItem[] response = retailer.submitOrder(items, customerDetails);

		if (response.length != 1)
		{
			throw new Exception("Response expected to contain exactly 1 item, contained " + response.length);
		}

		if (response[0].getQuantity().intValue() != 0)
		{
			throw new Exception("expected quantity to be 0, got " + response[0].getQuantity());
		}

		if (response[0].getPrice().doubleValue() != 0.0)
		{
			throw new Exception("expected price to be 0.0, got" + response[0].getPrice());
		}
	}
}