package org.wsi.scm.test;

import java.rmi.RemoteException;

import javax.naming.Context;

import org.w3c.dom.Element;
import org.wsi.scm.core.bo.configuration.ConfigurationType;
import org.wsi.scm.proxy.warehouse.WarehouseService;
import org.wsi.scm.proxy.callback.WarehouseCallbackViDocument;
import org.wsi.scm.proxy.callback.types.p1.Item;
import org.wsi.scm.util.Constants;
import org.wsi.scm.util.DemoConfiguration;
import org.wsi.scm.util.ServiceLocator;
import org.wsi.scm.util.Util;

import com.sap.engine.services.webservices.jaxrpc.exceptions.XmlUnmarshalException;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.BaseGeneratedStub;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.ProtocolList;
import com.sap.engine.services.webservices.jaxrpc.wsdl2java.features.builtin.SoapHeadersProtocol;

public class WarehouseCallbackServiceTest extends AbstractTest
{

	
	public String getTestName()
	{
		return ("WarehouseCallbackServiceTest");
	}
	
	public String getDescription()
	{
		return ("Tests the Warehouse (A) Callback Service submitSN operation");  
	}

	public void doTest() throws Exception
	{
		String demoUserId = "abc123";
		
		DemoConfiguration.getInstance().storeHttpProxyHostForDemo(demoUserId, "localhost");
		DemoConfiguration.getInstance().storeHttpProxyPortForDemo(demoUserId, "50012");	

		Context ctx = ServiceLocator.getInstance().getContext();

		WarehouseService ws =
			(WarehouseService) ctx.lookup(Constants.JNDI_SERVICE_PREFIX + Constants.WSPROXY_WAREHOUSE_SERVICE);
		WarehouseCallbackViDocument wha = (WarehouseCallbackViDocument) ws.getLogicalPort("WHAMFASecurePort");

		// get configuration from demo user id
		ConfigurationType configuration = DemoConfiguration.getInstance().getConfigurationForDemo(demoUserId);

		// create headers for callback operation
		Element configurationSoapHeader = Util.getConfigurationHeader(configuration);		
		Element callbackSoapHeader = Util.getCallbackHeader();

		// workaround for re-ordering of protocols needed to enable security for SOAP headers! 
		ProtocolList protocolList = ((BaseGeneratedStub) wha)._getGlobalProtocols();
		SoapHeadersProtocol whaRequestHeaders =
			(SoapHeadersProtocol) ((BaseGeneratedStub) wha)._getGlobalProtocols().getProtocol(
				"SoapHeadersProtocol");
		protocolList.remove(whaRequestHeaders);
		protocolList.add(0, whaRequestHeaders);

		// add headers
		whaRequestHeaders.setOutputHeader(null, callbackSoapHeader);
		whaRequestHeaders.setOutputHeader(null, configurationSoapHeader);
		 
		wha._setProperty("javax.xml.rpc.http.proxyhost", "localhost");
		wha._setProperty("javax.xml.rpc.http.proxyport", "50012");

		wha._setProperty(
		WarehouseCallbackViDocument.ENDPOINT_ADDRESS_PROPERTY,
			Util.getEndpointForRole(configuration, Constants.ROLE_WAREHOUSE_A));

		Item[] mItems = new Item[1];
		Item mItem = new Item();
		mItem.setID(new Long(605002));
		mItem.setQty(new Integer(2));
		mItem.setPrice(new Float("100.2"));
		mItems[0] = mItem;
		
		try
		{
			Boolean response = wha.submitSN("1", "1", "ABC999999EFG", mItems, new Float("1.9"));
		}
		catch (RemoteException e)
		{					
			if (e.getCause() instanceof XmlUnmarshalException)
			{
				e.printStackTrace();
			}
			else
				throw new RemoteException(e.getMessage()); 
		}
	}

}
